import QUnit from 'steal-qunit';
import { ViewModel } from './donejs-typeahead';
import F from 'funcunit';
F.attach(QUnit);

// ViewModel unit tests
QUnit.module('donejs-typeahead');

var exampleData = {
  my: 'my own value',
  myBoolean: true,
  myNumber: 1.23,
  notDefined: undefined
};


F.speed = 100;
F.timeout = 1000 * 60;


QUnit.module('GUI Interaction');
// remember to set debug on
QUnit.test('select an element / testuser', function() {
  F.open("//../src/donejs-typeahead.html");
  F('.donejs-typeahead-input').visible(function() {
    // @todo create issue in funcunit to make this
    // work correctly when using a class name instead
    // of an id
    var typeaheadInputForm = F('#temp-id-class-fails-to-set-focus');
    // var typeaheadInputForm = F('input.donejs-typeahead-input');
    F.speed = 200;
    var word = 'testus';
    typeaheadInputForm.type(word);
    // not sure why but you need to press down twice in order to make it work
    F('.donejs-typeahead-suggestion').exists().then(function() {
      ok(true, "suggestion is visible");
      typeaheadInputForm.type('[down]');
      typeaheadInputForm.type('[down]');
      typeaheadInputForm.type('[enter]');
    });
    F('.donejs-typeahead-selected-value').html('I love tests').then(function() {
      ok(true, "The value was set correcly to I love tests");
      F.speed = 100;
    });
  });
});

QUnit.test('select an element / nriesco', function() {
  F.open("//../src/donejs-typeahead.html");
  F('.donejs-typeahead-input').visible(function() {
    // @todo create issue in funcunit to make this
    // work correctly when using a class name instead
    // of an id
    var typeaheadInputForm = F('#temp-id-class-fails-to-set-focus');
    // var typeaheadInputForm = F('input.donejs-typeahead-input');
    F.speed = 200;
    var word = 'ries';
    typeaheadInputForm.type(word);
    // not sure why but you need to press down twice in order to make it work
    F('.donejs-typeahead-suggestion').exists().then(function() {
      ok(true, "suggestion is visible");
      typeaheadInputForm.type('[down]');
      typeaheadInputForm.type('[down]');
      typeaheadInputForm.type('[enter]');
    });
    F('.donejs-typeahead-selected-value').html('Nico').then(function() {
      ok(true, "The value was set correcly to Nico");
      F.speed = 100;
    });
  });
});



QUnit.module("ViewModel");
QUnit.test('Changing the default values', function() {
  var vm = new ViewModel({
    remoteHost: '127.0.0.1',
    displayValue: 'name'
  });
  QUnit.equal(vm.attr('remoteHost'), '127.0.0.1', 'remoteHost property changed');
  QUnit.equal(vm.attr('displayValue'), 'name', 'displayValue property changed');

});



QUnit.module("#restApiSearchResultToArray(data)");
QUnit.test('data is not valid', function() {
  var vm = new ViewModel();
  QUnit.deepEqual(vm.restApiSearchResultToArray(undefined), [], 'Undefined');
  QUnit.deepEqual(vm.restApiSearchResultToArray(null), [], 'null');
  QUnit.deepEqual(vm.restApiSearchResultToArray(false), [], 'false');
  QUnit.deepEqual(vm.restApiSearchResultToArray(-1), [], '-1');
  QUnit.deepEqual(vm.restApiSearchResultToArray(), [], 'empty');
});

QUnit.test('data.data is not valid', function() {
  var vm = new ViewModel();
  QUnit.deepEqual(vm.restApiSearchResultToArray({ a: 1 }), [], 'Undefined');
  QUnit.deepEqual(vm.restApiSearchResultToArray({ data: {} }), [], 'empty object');
});

QUnit.test('the result obtained is not in the expected format ( {data: [object1, object2]} )', function() {
  var vm = new ViewModel();
  QUnit.deepEqual(vm.restApiSearchResultToArray({ a: 1 }), [], 'no data field');
  QUnit.deepEqual(vm.restApiSearchResultToArray({ data: 1 }), [], 'data is not an array');
  QUnit.deepEqual(vm.restApiSearchResultToArray({ data: 'hello' }), [], 'data is not an array');
  QUnit.deepEqual(vm.restApiSearchResultToArray([{ some: 'data' }]), [], 'not an object');
});



QUnit.module("#replaceSpecialTokens(dirtyString)");
QUnit.test('wrong params', function() {
  var vm = new ViewModel();
  QUnit.equal(vm.replaceSpecialTokens(null), '', 'null');
  QUnit.equal(vm.replaceSpecialTokens({}), '', 'empty object');
  QUnit.equal(vm.replaceSpecialTokens({ my: "object" }), '', 'any object');
});

QUnit.test('no instances', function() {
  var vm = new ViewModel();
  var text = '<some>text no tokens</some>';
  QUnit.equal(vm.replaceSpecialTokens(text), text, 'no instances');
  text = '<some///{ {  { [ [   [   [ [ [  ]  ] ]     ]  ]  ]  } }';
  QUnit.equal(vm.replaceSpecialTokens(text), text, 'no instances');
});

QUnit.test('exactly one instance', function() {
  var vm = new ViewModel();
  var text = '';
  text = '<some>text with one token [[</some>';
  QUnit.equal(vm.replaceSpecialTokens(text), '<some>text with one token {{</some>', 'one instance');
  text = '<some>text with one token ]]</some>';
  QUnit.equal(vm.replaceSpecialTokens(text), '<some>text with one token }}</some>', 'one instance');
  text = '<some>text with one token [[[</some>';
  QUnit.equal(vm.replaceSpecialTokens(text), '<some>text with one token {{[</some>', 'one instance');
  text = '<some>text with one token ]]]</some>';
  QUnit.equal(vm.replaceSpecialTokens(text), '<some>text with one token }}]</some>', 'one instance');
});

QUnit.test('more than one instance', function() {
  var vm = new ViewModel();
  var text = '';
  text = '<some>text [[with]] several [[tokens]] </some>';
  QUnit.equal(vm.replaceSpecialTokens(text), '<some>text {{with}} several {{tokens}} </some>', 'text containing several tokens');
});



// @todo move to utils
QUnit.module("#getSelectedElementByKey(key)");
QUnit.test('Retrieve standard types', function() {
  var vm = new ViewModel();
  vm.attr('selectedElementObject', exampleData);
  QUnit.equal(vm.getSelectedElementByKey('my'), 'my own value', 'String value successfully retrieved');
  QUnit.equal(vm.getSelectedElementByKey('myBoolean'), true, 'Boolean value successfully retrieved');
  QUnit.equal(vm.getSelectedElementByKey('myNumber'), 1.23, 'Number value successfully retrieved');
});

QUnit.test('Try to retrieve non existing or invalid types', function() {
  var vm = new ViewModel();
  vm.attr('selectedElementObject', exampleData);
  QUnit.equal(vm.getSelectedElementByKey('undefinedKey'), null, 'Key does not exist');
  QUnit.equal(vm.getSelectedElementByKey('notDefined'), null, 'Key does not exist');

});
